/*
 * (C) Copyright 2004
 * Pantelis Antoniou, Intracom S.A., panto@intracom.gr
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Password functions & factory defaults
 */
#include <common.h>
#include <command.h>

/* Note no controlling CFG_CMD_xxx; it is enterily board controlled */

#ifdef CONFIG_FACTORY_DEFAULTS

extern void factory_defaults(void);

static int prompt_factory_defaults(void)
{
	char c;

	puts("WARNING: Factory settings will be restored; Confirm [Y/n] ");
	c = getc();
	putc(c);
	putc('\n');
	if (c != 'Y' && c != 'y')
		return 1;

	puts("Restoring factory defaults.... ");
	factory_defaults();
	puts("\n");

	return 0;
}

int do_factory(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	return prompt_factory_defaults();
}

U_BOOT_CMD(
	factory,    1,    1,     do_factory,
	"factory - restore factory defaults\n",
	NULL
);
#endif

#ifdef CONFIG_PASSWORD_ACCESS

static int i64c(int i)
{
	if (i <= 0)
		return ('.');
	if (i == 1)
		return ('/');
	if (i >= 2 && i < 12)
		return ('0' - 2 + i);
	if (i >= 12 && i < 38)
		return ('A' - 12 + i);
	if (i >= 38 && i < 63)
		return ('a' - 38 + i);
	return ('z');
}

static char *make_salt(void)
{
	DECLARE_GLOBAL_DATA_PTR;
	bd_t *bd = gd->bd;
	static unsigned long x;
	static char result[3];

	/* make semi random salt */
	x += get_timer(0) + bd->bi_baudrate +
		bd->bi_enetaddr[3] + bd->bi_enetaddr[4] + bd->bi_enetaddr[5];
	result[0] = i64c(((x >> 18) ^ (x >> 6)) & 077);
	result[1] = i64c(((x >> 12) ^ x) & 077);
	result[2] = '\0';
	return result;
}

static char my_getc(int max_time)
{
    int delay, i;
    
    if (!max_time)
	return getc();
	
    for (delay = 0; delay < max_time; delay++) {
	for (i = 0; i < 1000; i++) {
	    if (tstc())
		return getc();	/* check for chars every 1mS */
    	    udelay(1000);
	}
    }
    return '\r';

}

static const char *ask_pass(int max_time)
{
	int c;
	static char buf[10];	/* only 8 chars significant */
	char *s, *e;

	s = buf;
	e = buf + sizeof(buf) - 1;
	c = my_getc(max_time);
	while ((c != '\r') && (c != '\n')) {
		switch (c) {
			case '\b':	/* backspace */
			case 0x7f:	/* rubout */
				if (s > buf) {
					puts("\b \b");	/* erase star */
					s--;
				}
				break;
			case 'U' - 'A' + 1:	/* Ctrl-U erase all */
				while (s > buf) {
					puts("\b \b");	/* erase star */
					s--;
				}
				break;
			default:
				if (s >= e)	/* no more */
					break;
				*s++ = (char)c;
				putc('*');
				break;
		}
		c = my_getc(max_time);
	}
	putc('\n');
	*s = '\0';

	return buf;
}

int do_passwd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char pass[20];
	const char *s;

	puts("Enter new password: ");
	s = ask_pass(0);
	strcpy(pass, s);
	puts("Confirm new password: ");
	s = ask_pass(0);
	if (strcmp(s, pass) != 0) {
		puts("Confirmation failed.\n");
		return 1;
	}
	/* clearing password? */
	if (pass[0] == '\0') {
		setenv("password", NULL);
		return 0;
	}

	/* and set it */
	setenv("password", (char *)md5_crypt(pass, make_salt()));
	
	puts("Note: To survive reboot do a 'saveenv'\n");

	return 0;
}

U_BOOT_CMD(
	passwd ,    1,    1,     do_passwd,
	"passwd  - Change access password\n",
	NULL
);

const char *get_stored_password(void)
{
	const char *stored;

	stored = getenv("password");
	if (stored == NULL)	/* no password; default is access permitted */
		return NULL;

	return stored;
}

int password_access(void)
{
	const char *stored, *encrypted, *unencrypted; 
	int j = 0;

	stored = get_stored_password();
	if (stored == NULL)	/* no password; default is access permitted */
		return 0;

//	puts("\nEnter password\n"
//#ifdef CONFIG_FACTORY_DEFAULTS
//		"  or enter 'factory' to reset to defaults\n"
//#endif
//		"\n");

	for (;;) {
		puts("Password: ");
		unencrypted = ask_pass(CONFIG_PASSWORD_TIMEOUT);
		if (unencrypted && unencrypted[0] != '\0') {

#ifdef CONFIG_FACTORY_DEFAULTS
			if (strcmp(unencrypted, "factory") == 0) {
				if (prompt_factory_defaults() == 0) {
					/* and re-read password */
					stored = get_stored_password();
					if (stored == NULL)	/* no password; default is access permitted */
						return 0;
				}
				continue;
			}
#endif
			encrypted = md5_crypt(unencrypted, stored);
			if (strcmp(encrypted, stored) == 0)
				break;
		}

		if (unencrypted[0] != '\0')
		    puts("Password incorrect\n");
		/* allow 3 consecutive attempts */
		if (++j >= 3) {
		    return -1;
		}
	}

	/* access granted */
	puts("\n");
	return 0;
}

#endif
